/*
	
	RLFMain.cpp

	(c) 1998-2001 Robert Lanziner-Furtenbach
	
*/

#include "RLFDebug.h"
#include "RLFMain.h"
#include "RLFWindow.h"
#include "RLFSetting.h"

#include <Alert.h>

const char *APP_SIG		= "application/x-vnd.RLF-CodeWalker";

BRect DefaultRect ( 60, 60, 499, 399 );

int main()															// the main procedure
{
	DEBUG_PRINT ( "Main" );
	RLFApp app;														// Create an application instance
	app.Run();														// Run the application looper
	return(0);														// exit the program
}

RLFApp::RLFApp ()													// RLFApplication constructor
	: BApplication(APP_SIG), WRect ( DefaultRect ), OpenPanel ( NULL )
{
	DEBUG_PRINT ( "RLFApp constructor" );
	DefaultSettings ( &DefaultMsg );								// set default preferences into DefaultMsg
	LoadSettings ( &ValueMsg );										// try to load preference-file (msg) into ValueMsg
}

RLFApp::~RLFApp ()													// RLFApplication destructor
{
	DEBUG_PRINT ( "RLFApp destructor" );
	if ( OpenPanel == NULL ) delete OpenPanel;						// get rid off the OpenPanel if necessary
	SaveSettings ( &ValueMsg );										// save settings to disk
}


void RLFApp::MessageReceived ( BMessage *message )					// overriding MessageReceived
{
	BWindow	*window;
	int32	i = 0;
	DEBUG_PRINT ( "RLFApp MessageReceived" );
	DEBUG_EXEC ( message->PrintToStream() );

	switch ( message->what ){
		case File_New:
			NewWindow ( NULL );										// create empty window
			break;
		case File_Open:												// show Open Panel
			if ( OpenPanel == NULL ) {
				OpenPanel = new BFilePanel;
			}
			OpenPanel->Show();
			break;
		case Extra_Settings:
			new RLFSetting ( DefaultRect, &ValueMsg, &DefaultMsg );
			break;
		case Msg_Unreg_Win:
			if ( WinCnt > 0 ) WinCnt--;								// one text-window closes
			if ( WinCnt == 0 ) PostMessage ( B_QUIT_REQUESTED );	// ahhh, last text-window, tell myself to quit
			break;
		case Msg_Preferences:										// send Preferences to all windows
			ValueMsg = *message;
			while ( ( window = be_app->WindowAt(i++) ) != NULL ) {	// get next window
				if ( window->Lock() ) {
					window->PostMessage ( message );				// post preference to window
					window->Unlock();
				}
			}
			break;
		default:
			BApplication::MessageReceived ( message );				// forward message to BApplication
			break;
	}
}

void RLFApp::AboutRequested ()										// Copyright/left Info
{
	BAlert *Alert = new BAlert ( "About...", "\nCodeWalker v 1.2\n\n(c)1998-2001 Robert Lanziner-Furtenbach\nhttp://www.profisoftware.com\nmail to: rlf@profisoftware.com", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_WARNING_ALERT );
	Alert->Go(NULL);
}


void RLFApp::RefsReceived(BMessage *message)						// overiding BApplication::RefsReceived
{
	uint32		type;
	int32		count;
    entry_ref	ref;

	DEBUG_PRINT ( "RLFApp RefsReceived" );
	DEBUG_EXEC ( message->PrintToStream() );
    
	message->GetInfo ( "refs", &type, &count );						// test if correct message and get count of refs
	if ( type != B_REF_TYPE)
		return;
	for ( long i = --count; i >= 0; i-- ) {							// open a window for each correct refs
	    if ( message->FindRef("refs", i, &ref) == B_OK ){
	    	NewWindow ( &ref );     	
        }
    }
}

void RLFApp::ArgvReceived ( int32 argc, char **argv )				// overiding BApplication::ArgvReceived
{
	entry_ref	ref;
	for ( int32 i = 1; i < argc; i++ ) {							// open a window for each correct argv
		DEBUG_PRINTF ( ( "ARGV[%2li]=%s\n", i, argv[i] ) );
		if ( get_ref_for_path ( argv[i], &ref ) == B_OK )
			NewWindow ( &ref );
    }
}

void RLFApp::ReadyToRun	( void )									// when it is time to start working
{
	if ( CountWindows() == 0 ) NewWindow ( NULL );					// if no TextWindow open then create an empty one
}


void RLFApp::NewWindow ( entry_ref *ref )							// opens a Window with a file
{
	new RLFWindow ( WRect, ref, ++WinCnt, &ValueMsg );				// open a window at the posistion WRect with file based on ref, ...
	if ( WinCnt % 16 == 0 ) {										// reset the position every 16 Windows
		WRect = DefaultRect;
	} else {
		WRect.OffsetBy ( 16, 16 );									// inc the position for the next window 
	}
}
